# i.MX High Assurance Boot (Secure Boot)

meta-tq provides some support for building signed bootloader images for the
High Assurance Boot (HAB) feature of i.MX platforms. HAB allows to verify the
booted image against a public key hash programmed into the fuses of the i.MX
SoC to prevent manipulation of the software on the boot medium.

i.MX HAB exists in two variants: HAB4 (HAB Version 4) and AHAB (Advanced HAB).
Please refer to the HAB documentation, in particular the [i.MX Code Signing Tool
documentation](https://gitlab.apertis.org/pkg/imx-code-signing-tool/-/tree/apertis/v2024dev2/docs)
and documentation provided with U-Boot, [HAB4](https://source.denx.de/u-boot/u-boot/-/tree/master/doc/imx/habv4)
or [AHAB](https://source.denx.de/u-boot/u-boot/-/tree/master/doc/imx/ahab)
respectively, for further information on signing key generation, signature
creation and general concepts of HAB.

## Supported platforms

meta-tq supports HAB4 signature generation on all i.MX8M-based platforms:

- TQMa8MPxL
- TQMa8Mx
- TQMa8MxML
- TQMa8MxNL

meta-tq supports AHAB signature generation on TQMa93xx.

## Abbreviations

|     | Definition            |
| --- | --------------------- |
| CSF | Command Sequence File |
| CST | Code Signing Tool     |
| HAB | High Assurance Boot   |

## Configuration

Signature generation can either be handled as part of the Yocto build (for
convenience during development) or separately (strongly recommended for
production images, so signing keys can be stored on a protected host separate
from the build system).

In both cases, the "secure" distro feature must be enabled, either in a
custom distro config or in `local.conf`:

    DISTRO_FEATURES:append = " secure"

Enabling the distro feature has two effects:

- HAB support is enabled in U-Boot. The SPL will validate the BL3 boot stage,
  and various commands like `hab_status`/`ahab_status` are made available
  from the U-Boot command line.
- Optionally, [automated signing](#automated-signing) can be enabled using the
  `IMX_HAB_KEY_NAME` variable. If this variable is unset, signing will be
  skipped and a log message is printed.

### Separate signing step

If signatures are not generated as a part of the Yocto build, the
`IMX_HAB_KEY_NAME` variable should be left unset.

On supported platforms, the build will place the following files in the artifact
directory at `${DEPLOY_DIR_IMAGE}` to facilitate the creation of signatures (for
each U-Boot configuration `config` and boot source `target`):

- `habinfo-${config}.env-${target}`

  A shell environment file which contains information about HAB blocks and CSF
  offsets. The HAB block list must be copied into a CSF input file, which is
  then used by the Code Signing Tool.

  On HAB4 platforms, the autogenerated file might look like the following:

      SPL_CSF_OFF="0x29000"
      SPL_HAB_BLOCK="0x911fc0 0x0 0x29000"
      SLD_CSF_OFF="0x59020"
      SLD_HAB_BLOCK="0x401fcdc0 0x58000 0x1020"
      FIT_HAB_BLOCK="0x40200000 0x5B000 0xB3268
      0x402B3268 0x10E268 0x74E8
      0x960000 0x115750 0xB1B0
      "

  Two separate signatures need to be generated on these platforms: one for the
  SPL/BL2 (signing `SPL_HAB_BLOCK`) and one for the BL3 boot stage
  containing U-Boot, ATF and optionally OP-TEE (signing both `SLD_HAB_BLOCK`
  and `FIT_HAB_BLOCK`). `SPL_CSF_OFF` and `SLD_CSF_OFF` specify the offsets
  where the signatures need to be patched into the bootstream
  (`imx-boot-${MACHINE}-${config}.bin-${target}`).

  On AHAB platforms, the autogenerated file might look like the following:

      CONTAINER_HEADER_OFF="0x0"
      SIGNATURE_BLOCK_OFF="0x110"

  As for HAB4, two signatures are needed. But for AHAB they are generated in
  two separate signing steps. In the first step, a container with U-Boot, ATF
  and optionally OP-TEE is signed. In the next step, the whole bootstream with
  the signed container from step one, SPL and various firmware files is built
  using imx-mkimage and then signed again. Therefore two different `habinfo`
  files for each `config` are in `${DEPLOY_DIR_IMAGE}`.

- `csf_*-${config}.txt-${target}`

  Example CSF input files generated based on the data in the habinfo
  file. The templates that these files are based on are located under
  `meta-tq/dynamic-layers/freescale-layer/recipes-bsp/imx-mkimage/files`.
  A bbappend for the `imx-boot-tq` recipe can be used to customize these
  templates.

### Automated signing

For convenience during development, the entire signature generation can be
handled as a part of the Yocto build process. This requires the signing keys
to be provided by the `imx-cst-keys` package
(`meta-tq/dynamic-layers/freescale-layer/recipes-security/imx-cst-keys`).
A set of example keys is already included in meta-tq; other keys can be added
using a bbappend file.

To enable automated signing, the `IMX_HAB_KEY_NAME` variable must be set in a
distro config or in `local.conf`, for example:

    # This selects the example key for SoCs with HAB4 support
    IMX_HAB_KEY_NAME = "hab4"

With this configuration, signatures will be created based on the generated
CSF files and patched into the corresponding bootstream files
(`imx-boot-${MACHINE}-${config}.bin-${target}`).

## Limitations

The U-Boot configurations provided by meta-tq do not perform any verification
for the kernel and the root filesystem. Therefore, only U-Boot itself and the
other BL3 components (ATF and OP-TEE) are secured against manipulation by
default when HAB is enabled.

There are a number of options to extend the chain of trust beyond U-Boot:

- A kernel image in FIT format can be extended with a HAB Image Vector Table and
  then be signed using the same key used for U-Boot. Verification of the image
  can then be performed from U-Boot using `hab_auth_img` or similar commands
- The generic `uboot-sign` feature of Yocto allows to verify a signed FIT
  image against a public key embedded in U-Boot
- A small root filesystem can be embedded into the kernel FIT image as an
  initramfs, so no separate verification is necessary
- For the protection of a larger root filesystem, various solutions exist in the
  Linux kernel, each with different capabilities and guarantees:
  - DMVerity
  - DMIntegrity
  - DMCrypt
  - Integrity Measurement Architecture (IMA)

Discussion of the specific advantages and disadvantages of the different
approaches is out of scope for this document.
